searchNodes=[{"ref":"TwoFactorInACan.html","title":"TwoFactorInACan","module":"TwoFactorInACan","type":"module","doc":"TwoFactorInACan provides easy to use functions for implementing two factor authentication in a phoenix application."},{"ref":"TwoFactorInACan.Hotp.html","title":"TwoFactorInACan.Hotp","module":"TwoFactorInACan.Hotp","type":"module","doc":"Provides functions for working with the HMAC-based One Time Password algorithm as defined in RFC 4226. For details on RFC 4226, see https://tools.ietf.org/rfc/rfc4226.txt."},{"ref":"TwoFactorInACan.Hotp.html#generate_token/3","title":"TwoFactorInACan.Hotp.generate_token/3","module":"TwoFactorInACan.Hotp","type":"function","doc":"Generates a token from a shared secret and a counter which can be synchronized. This token can be used by one party to verify whether another party has the same secret. Options: :secret_format - the format that the secret is passed in as. Options include: :binary (default) :base32 :base64 :token_length (Default: 6) - the length of the generated token. A longer token is harder to guess and thus more secure. A longer token can also be more difficult for users to accurately transmit. Although everything in TwoFactorInACan supports variable token length, you should be sure that other apps and programs used support the token length set here. Examples iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret() iex&gt; TwoFactorInACan.Hotp.generate_token(secret, 0) &quot;866564&quot; iex&gt; TwoFactorInACan.Hotp.generate_token(secret, 1) &quot;532769&quot; iex&gt; TwoFactorInACan.Hotp.generate_token(secret, 0, token_length: 10) &quot;1807866564&quot;"},{"ref":"TwoFactorInACan.Hotp.html#same_secret?/4","title":"TwoFactorInACan.Hotp.same_secret?/4","module":"TwoFactorInACan.Hotp","type":"function","doc":"Verifies that the provided HOTP token was generated using the provided secret and count. This function uses the secret and count to generate a token. It then compares that generated token to the passed in token. If they match, then this function returns true. If they do not match, then this function returns false. This function allows a number of options: :secret_format - the format that the secret is passed in as. Options include: :binary (default) :base32 :base64 :token_length (Default: 6) - the length of the generated token. A longer token is harder to guess and thus more secure. A longer token can also be more difficult for users to accurately transmit. Although everything in TwoFactorInACan supports variable token length, you should be sure that other apps and programs used support the token length set here."},{"ref":"TwoFactorInACan.Secrets.html","title":"TwoFactorInACan.Secrets","module":"TwoFactorInACan.Secrets","type":"module","doc":"Functions for generating cryptographic secrets."},{"ref":"TwoFactorInACan.Secrets.html#generate_totp_secret/1","title":"TwoFactorInACan.Secrets.generate_totp_secret/1","module":"TwoFactorInACan.Secrets","type":"function","doc":"Generates a secret suitable for use in time based one time password (TOTP) two factor authentication. Generates a 160-bit key which is the size recommended by RFC4226 (https://www.ietf.org/rfc/rfc4226.txt). The following options are supported: :format (Default: :binary) - The format of the generated secret. Options include: :binary (default) :base32 :base64 :bytes (Default: 20) - The size in bytes of the generated secret. 20 bytes (160 bits) is the size specified by RFC 4226. It is also the size supported by most authenticator applications. Examples iex&gt; TwoFactorInACan.Secrets.generate_totp_secret() &lt;&lt;195, 110, 253, 36, 185, 138, 174, 16, 54, 176, 135, 67, 97, 11, 159, 63, 75, 80, 65, 6&gt;&gt; iex&gt; TwoFactorInACan.Secrets.generate_totp_secret(format: :base32) &quot;F2EJJEYSJA67QHI6DEAI2I6AGCEG7G5E&quot; iex&gt; TwoFactorInACan.Secrets.generate_totp_secret(format: :base64) &quot;xKXOSYcRVlHfnazLMlRinpb252U=&quot;"},{"ref":"TwoFactorInACan.Secrets.html#t:secret_generation_opts/0","title":"TwoFactorInACan.Secrets.secret_generation_opts/0","module":"TwoFactorInACan.Secrets","type":"type","doc":""},{"ref":"TwoFactorInACan.Totp.html","title":"TwoFactorInACan.Totp","module":"TwoFactorInACan.Totp","type":"module","doc":"Provides functions for working with time based one time password (TOTP) style two factor authentication (2FA) as defined in RFC 4226. Summary For details on RFC 4226, see https://tools.ietf.org/rfc/rfc4226.txt. TOTP two factor authentication uses the HMAC-based one time password (HOTP) algorithm with the current time passed in as the count to verify that the end user and the consuming application have the same secret. It does this by generating a token from the current time and secret and comparing the generated secret to the one the end user has supplied. If they match, then it is extremely unlikely that the end user does not have a different secret. Security Concerns When implementing 2FA using TOTP it is very important to also lock out any users which do not send a matching token some small number of consecutive times. By default tokens are 6 characters long which means that there is a 1/1,000,000 chance of guessing correctly. This is relatively trivial to brute force without some lock out mechanism. Care should also be token when adjusting the token length. Longer tokens are more secure, and shorter tokens are less secure (as they are easier to guess). If there is drift between clocks of users and the machine verifying tokens then the acceptable_past_tokens and acceptable_future_tokens options can be used to specify a window of acceptable tokens. This can also be useful to allow users to still submit tokens which were just seconds ago valid as a user experience enhancement. It should be noted that increasing the window of acceptable tokens also makes it easier to randomly guess a token. Care should be taken to way the pros and cons of this. As an example, by allowing one past token and one future token to be valid, there are now (usually) 3 tokens that an attacker could guess. This increases the chance of successfully guessing tokens to 1/333,333. Example Usage Secret Generation When a user first sets up TOTP 2FA, the server must generate a secret that will be shared between the server and the user. The TwoFactorInACan.Secrets module can be used for this. secret = TwoFactorInACan.Secrets.generate_totp_secret() &lt;&lt;109, 159, 180, 42, 128, 80, 183, 56, 163, 232, 151, 242, 233, 37, 167, 178, 253, 23, 18, 159&gt;&gt; Every user should have a different secret. No secrets should be shared. Transmitting the Secret to the User This secret must be securely transmitted to the user. It is extremely important that the method of transfer is secure. If an attacker can intercept the secret in transit, then they will be able to entirely bypass this form of 2FA. The attacker then only needs to acquire the user&#39;s password and this 2FA method is rendered useless. Using HTTPS is a must for transferring this secret. Do not transfer anything over HTTP! A common method of transferring the secret to an end user is by generating a QR code containing the secret. Smart phone applications such as Google Authenticator, Duo, and Authy can then scan these codes, extract the secret, and securely store the secret. For these applications to understand that a secret is being transmitted, the following url should be encoded to a QR Code: &quot;otpauth://totp/MyDescription?secret=MySecret&amp;issuer=MyAppName&quot; |&gt; MyQRModule.encode # Note: Not a real module! In the above URL, MySecret should be the secret encoded in base32, MyAppName and MyDescription can be anything, and assist the user in figuring out which token to use on your site if they have many. Verify User Received Secret At this point the end user should be asked to supply the current token as generated by their authenticator application. Upon receiving the token from the user, it should be verified against the secret generated earlier. TwoFactorInACan.Totp.same_secret?(generated_secret, user_supplied_token) true If the above function returns true then the the user has properly setup 2FA. If it returns false, then the incorrect token was sent. The user will need to send another correct token before their account should be configured to require TOTP 2FA at login. Store the secret The server will need to store the generated secret in some manner associated with the user so that it can be used to verify the user&#39;s identity at login. Care should be taken to ensure this is stored securely. If storing in a database, the database should be encrypted. Even if the database is encrypted, the field should be encrypted. You can even go the extra mile and use a different encryption key per user stored with the key stored outside of the database. If the secret is leaked or compromised by an attacker, then the attacker will be able to bypass this method of 2FA. In this event, the user should be prompted to re-setup TOTP 2FA. Ideally the user&#39;s account should be locked down until they can prove their identity through other means. Verify Token at Login When the user logs in, they should also supply the current TOTP token. The server should verify that the supplied TOTP token is generated using the same secret that is stored associated with that user. TwoFactorInACan.Totp.same_secret?(stored_user_secret, user_supplied_token) true If this returns true, then the user has sufficiently proven their identity and authentication should succeed (assuming they also supplied the correct password!) If this returns false, then login should fail. It can be a good practice to provide the user with a message that does not reveal whether the username didn&#39;t exist, the password was wrong, or 2FA failed. This makes an attackers job more difficult. There is some debate on whether this is an effective security measure."},{"ref":"TwoFactorInACan.Totp.html#current_token_value/2","title":"TwoFactorInACan.Totp.current_token_value/2","module":"TwoFactorInACan.Totp","type":"function","doc":"Outputs the current TOTP token for the given secret. Expects a 160-bit binary key by default. To use secrets that are encoded the :secret_format option can be supplied. It expects one of the following values: :binary (default) :base32 :base64 The following options are supported: :secret_format - the format that the secret is passed in as. Options include: :binary (default) :base32 :base64 :token_length (Default: 6) - the length of the generated token. A longer token is harder to guess and thus more secure. A longer token can also be more difficult for users to accurately transmit. Although everything in TwoFactorInACan supports variable token length, you should be sure that other apps and programs used support the token length set here. :offset_seconds (Default: 0) - The number of seconds to offset the current timestamp by. If the current timestamp was 600 and the offset was 60, then the timestamp of 660 would be used to calculate the time interval. This can be useful to account for drift or to purposefully allow the last token to be valid as well in functions that use this function. :interval_seconds (Default: 30) - The number of seconds that must pass before a new time_interval (and thus TOTP token) is returned by this function. This should probably never be anything but the default (30 seconds) during actual use, as nearly all apps that generate TOTP tokens assume a 30 second interval. :injected_timestamp (default: current time) - The unix timestamp to use when calculating the time interval. This should only be used during testing to ensure the same token or interval is always returned. When this option is not supplied, the current timestamp is used. Examples iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret() iex&gt; TwoFactorInACan.Totp.current_token_value(secret) &quot;858632&quot; iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret(format: :base32) iex&gt; TwoFactorInACan.Totp.current_token_value(secret, secret_format: :base32) &quot;743622&quot; iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret(format: :base64) iex&gt; TwoFactorInACan.Totp.current_token_value(secret, secret_format: :base64) &quot;384012&quot;"},{"ref":"TwoFactorInACan.Totp.html#same_secret?/3","title":"TwoFactorInACan.Totp.same_secret?/3","module":"TwoFactorInACan.Totp","type":"function","doc":"Verifies that the provided TOTP token was generated using the provided secret. This function uses the secret to generate a token. It then compares the generated token to the supplied token. If they match, then it can be probabilistically inferred that the entity that supplied the token also knew the secret. This function allows a number of options: :acceptable_past_tokens (Default: 0) - The number of past tokens which should result in this function returning true. Setting this to 1 can be a friendly way to allow users to still verify if they have taken too long to submit their token. It should be noted that this does result in two tokens being valid instead of one, which makes a valid token easeier to guess. This value should not be set too high or security is greatly compromised. :acceptable_future_tokens (Default: 0) - The number of future tokens which should result in this function returning true. It should be noted that setting this to a nonzero value will allow more tokens to be valid and thus make a valid token easier to guess. This value should not be set too high or security is greatly compromised. :token_length (Default: 6) - the length of the generated token. A longer token is harder to guess and thus more secure. A longer token can also be more difficult for users to accurately transmit. Although everything in TwoFactorInACan supports variable token length, you should be sure that other apps and programs used support the token length set here. :offset_seconds (Default: 0) - The number of seconds to offset the current timestamp by. If the current timestamp was 600 and the offset was 60, then the timestamp of 660 would be used to calculate the time interval. This can be useful to account for drift or difference in clock times between two entities. :interval_seconds (Default: 30) - The number of seconds that must pass before a new time_interval (and thus TOTP token) is returned by this function. This should probably never be anything but the default (30 seconds) during actual use, as nearly all apps that generate TOTP tokens assume a 30 second interval. :injected_timestamp (default: current time) - The unix timestamp to use when calculating the time interval. This should only be used during testing to ensure the same token or interval is always returned. When this option is not supplied, the current timestamp is used. :secret_format (default: :binary) - The format of the passed in secret. Can be one of :binary, :base32, or :base64. Examples iex&gt; secret = TwoFactorInACan.Secrets.generate_totp_secret() iex&gt; current_token = TwoFactorInACan.Totp.current_token_value(secret) iex&gt; TwoFactorInACan.Totp.same_secret?(secret, current_token) true"},{"ref":"TwoFactorInACan.Totp.html#time_interval/1","title":"TwoFactorInACan.Totp.time_interval/1","module":"TwoFactorInACan.Totp","type":"function","doc":"Calculates the current time interval as an integer used in the HOTP algorithm as the count to calculate the current token value. The following options are supported: :offset_seconds (Default: 0) - The number of seconds to offset the current timestamp by. If the current timestamp was 600 and the offset was 60, then the timestamp of 660 would be used to calculate the time interval. This can be useful to account for drift or to purposefully allow the last token to be valid as well in functions that use this function. :interval_seconds (Default: 30) - The number of seconds that must pass before a new time_interval (and thus TOTP token) is returned by this function. This should probably never be anything but the default (30 seconds) during actual use, as nearly all apps that generate TOTP tokens assume a 30 second interval. :injected_timestamp (default: current time) - The unix timestamp to use when calculating the time interval. This should only be used during testing to ensure the same token or interval is always returned. When this option is not supplied, the current timestamp is used. Examples iex&gt; TwoFactorInACan.Totp.time_interval() 51802243 iex&gt; TwoFactorInACan.Totp.time_interval(offset_seconds: 30) 51802244 iex&gt; TwoFactorInACan.Totp.time_interval(interval_seconds: 60) 25901122 iex&gt; TwoFactorInACan.Totp.time_interval(injected_timestamp: 1554067403) 51802246 iex&gt; TwoFactorInACan.Totp.time_interval(injected_timestamp: 60, interval_seconds: 10) 6"},{"ref":"getting-started.html","title":"Getting Started","module":"getting-started","type":"extras","doc":"Getting Started"},{"ref":"getting-started.html#summary","title":"Getting Started - Summary","module":"getting-started","type":"extras","doc":"TwoFactorInACan is a library for adding two factor authentication (2FA) to your application. Its goal is to be the most developer friendly way to securely setup two factor authentication in the Elixir/Erlang ecosystem. That means that not only is it thoroughly documented and easy to add to your project, but it&#39;s also the easiest to understand should you need to take a look behind the curtain."},{"ref":"getting-started.html#installation","title":"Getting Started - Installation","module":"getting-started","type":"extras","doc":"This library is not currently available on hex. Once it is: The package can be installed by adding two_factor_in_a_can to your list of dependencies in mix.exs: def deps do [ {:two_factor_in_a_can, &quot;~&gt; 0.1.0&quot;} ] end"},{"ref":"getting-started.html#usage","title":"Getting Started - Usage","module":"getting-started","type":"extras","doc":"Time-Based One Time Password (TOTP) See TwoFactorInACan.Totp for usage details. HMAC-Based One Time Password (HOTP) See TwoFactorInACan.Hotp for usage details."},{"ref":"roadmap.html","title":"Roadmap","module":"roadmap","type":"extras","doc":""},{"ref":"roadmap.html#roadmap","title":"Roadmap - Roadmap","module":"roadmap","type":"extras","doc":"This project is still very, very, very much under construction. That said, here&#39;s what&#39;s planned: [ ] HMAC based One Time Password 2FA (HOTP) [x] Secret Generation [x] Hotp.generate_token/3 function [x] Hotp.same_secret?/3 function [ ] Write documentation [x] Time-based One Time Password 2FA (TOTP) [x] Secret Generation [x] Totp.current_token_value/2 function [x] Totp.same_secret?/3 function [x] Write documentation [ ] Easy TOTP QR Code generation [ ] with nonce [ ] with sha256 [ ] Universal Second Factor (U2P) [ ] Add examples to README.md [ ] Tutorial for adding to Phoenix Maybe, but probably not 2FA via text to someone&#39;s phone (SMS) is better than no 2FA at all, but there are security users to be aware of. The main one is that it is not impossible to intercept the communication of the token when sent to the user&#39;s phone number. Ultimately, there are no cryptographic guarantees that the device used to authenticate is the same device that was originally used to setup 2FA. [ ] SMS based 2FA [ ] Thoroughly warn about not using [ ] But note why its still better than no 2FA at all [ ] Build OTP process tree for keeping state of recently sent SMS [ ] Sms.generate_and_send_token/2 function [ ] Sms.verify_token/3 function [ ] Document to pieces"}]